WINDOWS BUILD NOTES
===================

There are several options that can be use to build Bitcoin SV for Windows.

Using Microsoft Visual Studio 2019
-----------------------------

IMPORTANT: Compilers and libraries have subtle differences among platforms.
Please note that builds created with Visual Studio are currently not extensively
tested by QA team and are therefore not in scope for bug bounty programs.
Provided instructions are intended for developers that would like to use
refactoring and code analysis tools provided by Visual Studio and its associated tools.

### Installing dependencies

- Install Git
- Install Python 3 (Required for performing the build and running functional tests.)
- Install Visual Studio 2019
- Install CMake (https://cmake.org) (version >= 3.16)
- Install OpenSSL 1.0.2 (http://slproweb.com/products/Win32OpenSSL.html) - needed for functional tests. Do not forget to add OpenSSL bin directory to Windows path.

To build Bitcoin SV on Windows, you first need to install VCPKG package manager
- follow instructions at https://github.com/Microsoft/vcpkg - open "x64 Native
Tools Command Prompt for VS2019" and run the following:

```
C:
cd \
git clone --branch 2020.01 https://github.com/Microsoft/vcpkg.git
cd vcpkg
bootstrap-vcpkg.bat
```

After VCPKG is installed you can download and build dependencies by invoking the
following command:

```
vcpkg install --triplet x64-windows-static boost-filesystem boost-signals2 boost-test boost-program-options boost-interprocess boost-random boost-assign boost-uuid boost-multi-index libevent openssl zeromq berkeleydb
```

This will take approximately 20 minutes.

### Generating project files

Visual studio project files can be generated by invoking CMAKE (adjust VCPKG
path if necessary). Open "x64 Native Tools Command Prompt for VS2019" and run
the following __inside your Bitcoin SV root directory__:

```
mkdir build
cd build
cmake  -DCMAKE_TOOLCHAIN_FILE=c:\vcpkg\scripts\buildsystems\vcpkg.cmake  -G "Visual Studio 16 2019" -Ax64 -DVCPKG_TARGET_TRIPLET=x64-windows-static -DCMAKE_BUILD_TYPE=Release ..

```

The CMAKE_BUILD_TYPE parameter is required. Without it, VCPKG will put both
debug and release libraries on linkers search path. This will cause problems at
link time, since BerkleyDB uses same file name for debug and release libraries
(libdb48.lib).

Project files will be written to build directory.

### Building Bitcoin SV

-   Open solution file `build/BitcoinSV.sln` that has been generated by CMAKE.

-   Select appropriate build configuration that matches your CMAKE_BUILD_TYPE.
    (RelWithDebInfo is recommended for debugging a Release build.)

-   If you do not want to run tests as part of your build, unload projects whose
    names contain “check” and “test”, except “test_bitcoin”.

-   Choose Build/Build solution

### Running functional tests
Go to build directory you created in instructions above.

Since build directory can contain outputs for multiple build configuration, you
need to specify which one to test with `--buildconfig` parameter:

```
test\functional\test_runner.py --buildconfig RelWithDebInfo
```

### Limitations

-   bitcoin seeder is not included in the build, since it uses APIs, that are
    not cross platform

-   64 bit MSVC compiler does not support 128 bit integers or inline assembly.
    Therefore, secp2561k is build with scalar and field implementation that
    might exhibit different performance characteristics compared to Linux buid.

-   secp2561k benchmark tests are not included in the build, since they use APIS
    that are not cross platform.

-   Some leveldb unit test fail on Windows – this is known problem (see
    <https://github.com/bitcoin-core/leveldb/issues/9> and
    <https://github.com/bitcoin-core/leveldb/issues/8>

-   Some of the libraries provided by VCPKG use versioned library names.
    Specific versions of the libraries are referenced in bitcoind build script.
    If you upgrade VCPG it might also upgrade libraries and the bitcoind build
    script will be unable to find old versions. To solve this either downgrade
    VCPKG (git reset --hard 2020.01) or add new
    library version to bitcoind CMAKE scripts located in cmake\\modules. See
    <https://github.com/Microsoft/vcpkg/issues/1681> for details.

### Editing Python files on Windows
Editing a Python file on Windows will clear its executable flag, making it unsuable
on Linux. To restore the flag use:

```
git update-index --chmod=+x path/to/file
```
